@@ -46,6 +46,7 @@
 #include <QtCore/QJsonObject>
 #include <QtCore/QMimeData>
 #include <QtCore/QProcess>
+#include <QtCore/QRandomGenerator>
 #include <QtCore/QSettings>
 #include <QtCore/QSortFilterProxyModel>
 #include <QtCore/QStandardPaths>
@@ -90,9 +91,8 @@
 #include <pqTestUtility.h>
 #endif
 
-#ifdef Avogadro_ENABLE_RPC
-#include <molequeue/client/client.h>
-#endif // Avogadro_ENABLE_RPC
+// RPC functionality is provided by the local rpc library
+// The registerMoleQueue() function is currently disabled
 
 #ifdef AVO_USE_VTK
 #include <avogadro/vtk/vtkglwidget.h>
@@ -277,6 +277,9 @@ MainWindow::MainWindow(const QStringList& fileNames, bool disableSettings)
   // The default settings will be used if everything was cleared.
   readSettings();
 
+  // check for auto-save files
+  checkAutosaveRecovery();
+
   // check for version update
   checkUpdate();
 
@@ -446,7 +449,7 @@ void MainWindow::setupInterface()
   m_toolDock = new QDockWidget(tr("Tool"), this);
   addDockWidget(Qt::LeftDockWidgetArea, m_toolDock);
 
-  // Our scene/view dock.
+  // Our dock for toggling different display types.
   m_sceneDock = new QDockWidget(tr("Display Types"), this);
   m_sceneTreeView = new QTreeView(m_sceneDock);
   m_sceneTreeView->setIndentation(0);
@@ -456,8 +459,8 @@ void MainWindow::setupInterface()
   m_sceneDock->setWidget(m_sceneTreeView);
   addDockWidget(Qt::LeftDockWidgetArea, m_sceneDock);
 
-  // Our view dock.
-  m_viewDock = new QDockWidget(tr("View Configuration"), this);
+  // Our dock for configuring the display types.
+  m_viewDock = new QDockWidget(tr("Display Type Settings"), this);
   addDockWidget(Qt::LeftDockWidgetArea, m_viewDock);
 
   // Our molecule dock.
@@ -471,7 +474,7 @@ void MainWindow::setupInterface()
   m_moleculeDock->setWidget(m_moleculeTreeView);
   addDockWidget(Qt::LeftDockWidgetArea, m_moleculeDock);
 
-  // Our molecule dock.
+  // Our layer dock.
   m_layerDock = new QDockWidget(tr("Layers"), this);
   m_layerTreeView = new QTreeView(m_layerDock);
   m_layerTreeView->setIndentation(0);
@@ -681,10 +684,14 @@ void MainWindow::newMolecule()
 template<class T, class M>
 void setWidgetMolecule(T* glWidget, M* mol)
 {
+  // reset the camera when the previous molecule was empty
+  M* previousMol = glWidget->molecule();
+  bool needReset = (previousMol == nullptr || previousMol->atomCount() == 0);
   glWidget->setMolecule(mol);
   glWidget->updateScene();
-  if (mol->atomCount() == 0)
+  if (needReset) {
     glWidget->resetCamera();
+  }
 }
 
 void setDefaultViews(MultiViewWidget* viewWidget)
@@ -761,10 +768,14 @@ void MainWindow::setMolecule(Molecule* mol)
   if (oldMolecule)
     oldMolecule->disconnect(this);
 
+  // start the autosave timer
+  startAutosaveTimer();
+
   // Check if the molecule needs to update the current one.
   QWidget* w = m_multiViewWidget->activeWidget();
   if (auto* glWidget = qobject_cast<QtOpenGL::GLWidget*>(w)) {
     setWidgetMolecule(glWidget, mol);
+    glWidget->setFocus();
   }
 #ifdef AVO_USE_VTK
   else if (auto* vtkWidget = qobject_cast<vtkGLWidget*>(w)) {
@@ -1079,6 +1090,19 @@ bool MainWindow::openFile(const QString& fileName, Io::FileFormat* reader)
   return true;
 }
 
+void MainWindow::setDefaultFileDialogPath(const QString& path)
+{
+  if (path.isEmpty())
+    return;
+
+  QFileInfo info(path);
+  QString dir = info.isDir() ? path : info.absolutePath();
+
+  QSettings settings;
+  settings.setValue("MainWindow/lastOpenDir", dir);
+  settings.setValue("MainWindow/lastSaveDir", dir);
+}
+
 void MainWindow::backgroundReaderFinished()
 {
   QString fileName = m_threadedReader->fileName();
@@ -1174,9 +1198,126 @@ bool MainWindow::backgroundWriterFinished()
   m_threadedWriter = nullptr;
   m_progressDialog->deleteLater();
   m_progressDialog = nullptr;
+
+  // On successful save, remove any autosave files
+  if (success)
+    cleanupAutosaves(fileName);
+
   return success;
 }
 
+void MainWindow::cleanupAutosaves(QString fileName)
+{
+  if (fileName.isEmpty())
+    return;
+
+#ifndef NDEBUG
+  qDebug() << "Cleaning up autosaves for " << fileName;
+#endif
+
+  QString autosaveDirPath =
+    QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) +
+    "/autosave";
+  QDir autosaveDir(autosaveDirPath);
+  QStringList autosaveFiles =
+    autosaveDir.entryList(QStringList() << "*_autosave.cjson", QDir::Files);
+  // check if fileName is in autosaveFiles
+  for (const QString& file : autosaveFiles) {
+    if (file.contains(fileName)) {
+      // we don't need this anymore because the real file is saved
+      QFile::remove(autosaveDir.absoluteFilePath(file));
+      break;
+    }
+  }
+}
+
+void MainWindow::cleanupCurrentAutosave()
+{
+  if (!m_molecule)
+    return;
+
+  QString autosaveDirPath =
+    QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) +
+    "/autosave";
+  QDir autosaveDir(autosaveDirPath);
+  if (!autosaveDir.exists())
+    return;
+
+  QStringList autosaveFiles =
+    autosaveDir.entryList(QStringList() << "*_autosave.cjson", QDir::Files);
+
+  // Determine the search pattern based on whether the molecule has a filename
+  QString searchPattern;
+  if (m_molecule->hasData("fileName")) {
+    QFileInfo fileInfo(m_molecule->data("fileName").toString().c_str());
+    searchPattern = fileInfo.baseName();
+  } else {
+    // For untitled documents, match by formula
+    searchPattern = QString::fromStdString(m_molecule->formula());
+    searchPattern.remove(QRegularExpression("[^A-Za-z0-9]"));
+  }
+
+  if (searchPattern.isEmpty())
+    return;
+
+  for (const QString& file : autosaveFiles) {
+    if (file.contains(searchPattern)) {
+      QFile::remove(autosaveDir.absoluteFilePath(file));
+      // Don't break - there might be multiple autosaves for untitled documents
+    }
+  }
+}
+
+void MainWindow::checkAutosaveRecovery()
+{
+  QString autosaveDirPath =
+    QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) +
+    "/autosave";
+  QDir autosaveDir(autosaveDirPath);
+  if (!autosaveDir.exists())
+    return;
+
+  QStringList autosaveFiles =
+    autosaveDir.entryList(QStringList() << "*_autosave.cjson", QDir::Files);
+  if (autosaveFiles.isEmpty())
+    return; // great, nothing to do
+
+  QMessageBox::StandardButton reply = QMessageBox::question(
+    this, tr("Recover Autosaved Files"),
+    tr("Autosave files were found from a previous session.\n"
+       "Would you like to recover them?"),
+    QMessageBox::Yes | QMessageBox::No);
+
+  if (reply == QMessageBox::Yes) {
+    for (const QString& file : autosaveFiles) {
+      QString path = autosaveDir.absoluteFilePath(file);
+      openFile(path, new Io::CjsonFormat);
+    }
+  }
+
+  // Optionally, cleanup old autosaves
+  if (reply == QMessageBox::No) {
+    for (const QString& file : autosaveFiles)
+      QFile::remove(autosaveDir.absoluteFilePath(file));
+  }
+}
+
+void MainWindow::startAutosaveTimer()
+{
+  if (m_autosaveTimer)
+    return;
+
+  m_autosaveTimer = new QTimer(this);
+
+  QSettings settings;
+  int intervalMinutes = settings.value("autosave/interval", 2).toInt();
+  int intervalMs = qMax(1, intervalMinutes) * 60 * 1000;
+
+  connect(m_autosaveTimer, &QTimer::timeout, this,
+          &MainWindow::autosaveDocument);
+  m_autosaveTimer->start(intervalMs);
+}
+
 void MainWindow::toolActivated()
 {
   if (auto* action = qobject_cast<QAction*>(sender())) {
@@ -1648,7 +1789,8 @@ bool MainWindow::saveFile(bool async)
     else if (box.clickedButton() == cancelButton)
       return false;
     else if (box.clickedButton() == exportButton)
-      return exportFile(async);
+      return exportFile(QString ::fromStdString(fileName),
+                        async); // save in current format
   }
 
   // Otherwise, prompt for a new valid format.
@@ -1849,6 +1991,7 @@ void MainWindow::setActiveTool(QString toolName)
         m_toolDock->setWindowTitle(toolPlugin->activateAction()->text());
       }
     }
+    glWidget->setFocus();
   }
 
   if (!toolName.isEmpty()) {
@@ -1882,9 +2025,12 @@ void MainWindow::setActiveDisplayTypes(QStringList displayTypes)
   //  foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
   //    scene->setEnabled(false);
   foreach (ScenePlugin* scene, scenePluginModel->scenePlugins())
-    foreach (const QString& name, displayTypes)
+    foreach (const QString& name, displayTypes) {
       if (scene->objectName() == name)
         scene->setEnabled(true);
+      if (scene->name() == name)
+        scene->setEnabled(true);
+    }
 
   if (glWidget)
     glWidget->updateScene();
@@ -2182,8 +2328,14 @@ void MainWindow::buildMenu()
 #ifndef Q_OS_MAC
   action->setIcon(QIcon::fromTheme("document-export"));
 #endif
-  connect(action, &QAction::triggered, this,
-          static_cast<void (MainWindow::*)()>(&MainWindow::exportGraphics));
+  connect(action, SIGNAL(triggered()), this, SLOT(exportGraphics()));
+  // also add to the toolbar
+  action = new QAction(tr("Export Graphics…"), this);
+  m_fileToolBar->addAction(action);
+#ifndef Q_OS_MAC
+  action->setIcon(QIcon::fromTheme("document-export"));
+#endif
+  connect(action, SIGNAL(triggered()), this, SLOT(exportGraphics()));
 
   // Quit
   action = new QAction(tr("&Quit"), this);
@@ -2328,6 +2480,46 @@ void MainWindow::buildMenu()
   m_menuBuilder->buildMenuBar(menuBar());
 }
 
+void MainWindow::autosaveDocument()
+{
+  if (!m_molecule || !m_moleculeDirty) {
+    return; // No molecule loaded or no changes made since the last save.
+  }
+
+  QString autosaveDirPath =
+    QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) +
+    "/autosave";
+  QDir autosaveDir(autosaveDirPath);
+  if (!autosaveDir.exists()) {
+    autosaveDir.mkpath(".");
+  }
+
+  // Construct autosave file name
+  QString autosaveFileName;
+  if (m_molecule->hasData("fileName")) {
+    QFileInfo fileInfo(m_molecule->data("fileName").toString().c_str());
+    autosaveFileName = fileInfo.baseName() + "_autosave.cjson";
+  } else {
+    QString formula = QString::fromStdString(m_molecule->formula());
+    formula.remove(QRegularExpression("[^A-Za-z0-9]")); // sanitize
+    QString uid =
+      QString::number(QRandomGenerator::global()->generate(), 16).left(5);
+
+    autosaveFileName =
+      tr("untitled") + QString("%1_%2_autosave.cjson").arg(formula).arg(uid);
+  }
+
+  QString autosaveFilePath = autosaveDirPath + "/" + autosaveFileName;
+
+  // Use CJSON format for autosaving
+  Io::CjsonFormat writer;
+  if (!writer.writeFile(autosaveFilePath.toLocal8Bit().data(), *m_molecule)) {
+    qWarning() << "Failed to autosave the document to" << autosaveFilePath;
+  } else {
+    qDebug() << "Document autosaved to" << autosaveFilePath;
+  }
+}
+
 void MainWindow::buildMenu(QtGui::ExtensionPlugin* extension)
 {
   foreach (QAction* action, extension->actions())
@@ -2510,6 +2702,7 @@ bool MainWindow::saveFileIfNeeded()
         // fails:
         return saveFile(/*async=*/false);
       case QMessageBox::Discard:
+        cleanupCurrentAutosave();
         markMoleculeClean();
         return true;
       default:
