@@ -69,7 +69,7 @@ public:
 
 public slots:
   void setMolecule(Avogadro::QtGui::Molecule* molecule);
-
+  void autosaveDocument(); // Autosave the current document
   /**
    * Update internal state to reflect that the molecule has been modified.
    */
@@ -91,6 +91,13 @@ public slots:
    */
   bool openFile(const QString& fileName, Io::FileFormat* reader = nullptr);
 
+  /**
+   * Set the default directory for open/save dialogs.
+   * Called when opening files from the command line to respect the
+   * current working directory.
+   */
+  void setDefaultFileDialogPath(const QString& path);
+
   void exportGraphics(QString fileName);
 
   /**
@@ -136,6 +143,19 @@ public:
    */
   void readSettings();
 
+  /**
+   * Attempt to recover autosave files from a crash
+   */
+  void checkAutosaveRecovery();
+
+  void startAutosaveTimer();
+
+  QString autosaveFilePath() const;
+
+  void cleanupAutosaves(QString filename);
+
+  void cleanupCurrentAutosave();
+
   /**
    * Set the list of possible translations
    */
@@ -404,7 +424,7 @@ private:
   QtGui::ScenePlugin* m_activeScenePlugin;
   bool m_queuedFilesStarted;
   QStringList m_queuedFiles;
-
+  QTimer* m_autosaveTimer = nullptr; // for the autosave timer
   QStringList m_recentFiles;
   QList<QAction*> m_actionRecentFiles;
 
